#include "Visitor.h"

#include <ArmarXCore/core/exceptions/local/ExpressionException.h>

#include <RobotAPI/libraries/armem/core/workingmemory/Memory.h>
#include <RobotAPI/libraries/armem/core/error.h>


namespace armarx::armem::wm
{

    Visitor::Visitor()
    {
    }

    Visitor::~Visitor()
    {
    }

    bool Visitor::applyTo(Memory& memory)
    {
        bool cont = true;
        visitEnter(memory);
        for (auto& [_, coreSeg] : memory)
        {
            if (!applyTo(coreSeg))
            {
                cont = false;
                break;
            }
        }
        visitExit(memory);
        return cont;
    }

    bool Visitor::applyTo(CoreSegment& coreSegment)
    {
        bool cont = true;
        visitEnter(coreSegment);
        for (auto& [_, provSeg] : coreSegment)
        {
            if (!applyTo(provSeg))
            {
                cont = false;
                break;
            }
        }
        visitExit(coreSegment);
        return cont;
    }

    bool Visitor::applyTo(ProviderSegment& providerSegment)
    {
        bool cont = true;
        visitEnter(providerSegment);
        for (auto& [_, entity] : providerSegment)
        {
            if (!applyTo(entity))
            {
                cont = false;
                break;
            }
        }
        visitExit(providerSegment);
        return cont;
    }

    bool Visitor::applyTo(Entity& entity)
    {
        bool cont = true;
        visitEnter(entity);
        for (auto& [_, snapshot] : entity)
        {
            if (!applyTo(snapshot))
            {
                cont = false;
                break;
            }
        }
        visitExit(entity);
        return cont;
    }

    bool Visitor::applyTo(EntitySnapshot& snapshot)
    {
        bool cont = true;
        visitEnter(snapshot);
        for (auto& instance : snapshot)
        {
            if (!applyTo(instance))
            {
                cont = false;
                break;
            }
        }
        visitExit(snapshot);
        return cont;
    }

    bool Visitor::applyTo(EntityInstance& instance)
    {
        return visit(instance);
    }


    bool Visitor::applyTo(const Memory& memory)
    {
        bool cont = true;
        visitEnter(memory);
        for (const auto& [_, coreSeg] : memory)
        {
            if (!applyTo(coreSeg))
            {
                cont = false;
                break;
            }
        }
        visitExit(memory);
        return cont;
    }

    bool Visitor::applyTo(const CoreSegment& coreSegment)
    {
        bool cont = true;
        visitEnter(coreSegment);
        for (const auto& [_, provSeg] : coreSegment)
        {
            if (!applyTo(provSeg))
            {
                cont = false;
                break;
            }
        }
        visitExit(coreSegment);
        return cont;
    }

    bool Visitor::applyTo(const ProviderSegment& providerSegment)
    {
        bool cont = true;
        visitEnter(providerSegment);
        for (const auto& [_, entity] : providerSegment)
        {
            if (!applyTo(entity))
            {
                cont = false;
                break;
            }
        }
        visitExit(providerSegment);
        return cont;
    }

    bool Visitor::applyTo(const Entity& entity)
    {
        bool cont = true;
        visitEnter(entity);
        for (const auto& [_, snapshot] : entity)
        {
            if (!applyTo(snapshot))
            {
                cont = false;
                break;
            }
        }
        visitExit(entity);
        return cont;
    }

    bool Visitor::applyTo(const EntitySnapshot& snapshot)
    {
        bool cont = true;
        visitEnter(snapshot);
        for (const auto& instance : snapshot)
        {
            if (!applyTo(instance))
            {
                cont = false;
                break;
            }
        }
        visitExit(snapshot);
        return cont;
    }

    bool Visitor::applyTo(const EntityInstance& instance)
    {
        return visit(instance);
    }

}
