#pragma once

#include <RobotAPI/interface/armem/server/MemoryInterface.h>
#include <RobotAPI/interface/armem/client/MemoryListenerInterface.h>

#include <RobotAPI/libraries/armem/core/workingmemory/Memory.h>
#include <RobotAPI/libraries/armem/core/longtermmemory/Memory.h>
#include <RobotAPI/libraries/armem/client/Query.h>


namespace armarx::armem::server
{


    /**
     * @brief Helps connecting a Memory server to the Ice interface.
     *
     * This involves conversion of ice types to C++ types as well as
     * catchin exceptions and converting them to error messages
     */
    class MemoryToIceAdapter
    {
    public:

        /// Construct an MemoryToIceAdapter from an existing Memory.
        MemoryToIceAdapter(wm::Memory* workingMemory = nullptr, ltm::Memory* longtermMemory = nullptr);

        void setMemoryListener(client::MemoryListenerInterfacePrx memoryListenerTopic);


        // WRITING
        data::AddSegmentResult addSegment(
            const data::AddSegmentInput& input, bool addCoreSegments = false);

        data::AddSegmentsResult addSegments(
            const data::AddSegmentsInput& input, bool addCoreSegments = false);


        data::CommitResult commit(const data::Commit& commitIce, Time timeArrived);
        data::CommitResult commit(const data::Commit& commitIce);
        armem::CommitResult commit(const armem::Commit& commit);

        // READING
        query::data::Result query(const armem::query::data::Input& input);
        client::QueryResult query(const client::QueryInput& input);

        // LTM LOADING
        query::data::Result load(const armem::query::data::Input& input);

        // LTM STORING
        data::StoreResult store(const armem::data::StoreInput& input);

    public:

        wm::Memory* workingMemory;
        ltm::Memory* longtermMemory;

        client::MemoryListenerInterfacePrx memoryListenerTopic;

    };


}
