#pragma once

#include <string>

#include <QObject>

#include <ArmarXCore/interface/observers/ObserverInterface.h>
#include <ArmarXCore/core/logging/Logging.h>

#include <RobotAPI/interface/armem/mns/MemoryNameSystemInterface.h>
#include <RobotAPI/libraries/armem/client/Reader.h>
#include <RobotAPI/libraries/armem_gui/lifecycle.h>
#include <RobotAPI/libraries/armem_gui/instance/GroupBox.h>
#include <RobotAPI/libraries/armem_gui/memory/GroupBox.h>
#include <RobotAPI/libraries/armem_gui/PeriodicUpdateWidget.h>
#include <RobotAPI/libraries/armem_gui/LTMControlWidget.h>

class QBoxLayout;
class QDialog;
class QGroupBox;
class QLabel;
class QLayout;
class QSettings;


namespace armarx
{
    class ManagedIceObject;
    class SimpleConfigDialog;
}

namespace armarx::armem::gui
{

    class MemoryViewer :
        public QObject,
        public armarx::Logging,
        public armarx::gui::LifecycleClient
    {
        Q_OBJECT
        using This = MemoryViewer;


    public:

        MemoryViewer(
            QBoxLayout* updateWidgetLayout,
            QBoxLayout* ltmControlWidgetLayout,
            QGroupBox* memoryGroupBox, QLayout* memoryGroupBoxParentLayout,
            QGroupBox* instanceGroupBox, QLayout* instanceGroupBoxParentLayout,
            QLabel* statusLabel
        );

        void setLogTag(const std::string& tag);


        void loadSettings(QSettings* settings);
        void saveSettings(QSettings* settings);

        void writeConfigDialog(SimpleConfigDialog* dialog);
        void readConfigDialog(SimpleConfigDialog* dialog);


    public slots:

        // LTMControlWidget
        void store();
        void exportHere();

        // Other
        void updateMemories();
        void updateInstanceTree(const armem::MemoryID& selectedID);


    signals:

        void initialized();
        void connected();
        void disconnected();

        void memoryTreeUpdated();
        void instanceTreeUpdated();


    private slots:

        void updateMemoryTree();

    signals:

        void memoryDataChanged();


    private:

        void onInit(ManagedIceObject& component);
        void onConnect(ManagedIceObject& component);
        void onDisconnect(ManagedIceObject& component);


    public:

        std::string mnsName;
        armem::mns::MemoryNameSystemInterfacePrx mns;

        std::map<std::string, armem::client::Reader> memoryReaders;
        std::map<std::string, std::optional<armem::wm::Memory>> memoryData;

        QLayout* updateWidgetLayout = nullptr;
        armem::gui::PeriodicUpdateWidget* updateWidget = nullptr;

        QLayout* ltmControlWidgetLayout = nullptr;
        armem::gui::LTMControlWidget* ltmControlWidget = nullptr;

        armem::gui::MemoryGroupBox* memoryGroup = nullptr;

        armem::gui::InstanceGroupBox* instanceGroup = nullptr;

        QLabel* statusLabel = nullptr;


        std::string debugObserverName;
        DebugObserverInterfacePrx debugObserver;

        // Queries.
        armem::query::data::EntityQueryPtr entityQuery;

    };

}
