#include "DataTreeBuilder.h"

#include <QTreeWidgetItem>

#include <RobotAPI/libraries/armem_gui/TreeWidgetBuilder.h>


namespace armarx::armem::gui::instance
{

    DataTreeBuilder::DataTreeBuilder()
    {
    }

    void DataTreeBuilder::updateTree(QTreeWidgetItem* parent, aron::datanavigator::DictNavigator& data)
    {
        DictBuilder builder = getDictBuilder();
        builder.setUpdateItemFn([this, &data](const std::string & key, QTreeWidgetItem * item)
        {
            auto child = data.getElement(key);
            this->update(item, key, *child);
            return true;
        });

        builder.updateTree(parent, data.getAllKeys());
    }

    void DataTreeBuilder::updateTree(QTreeWidgetItem* parent, aron::datanavigator::ListNavigator& data)
    {
        auto children = data.getChildren();

        ListBuilder builder = getListBuilder();
        builder.setUpdateItemFn([this, &children](size_t key, QTreeWidgetItem * item)
        {
            this->update(item, std::to_string(key), *children.at(key));
            return true;
        });

        builder.updateTree(parent, getIndex(children.size()));
    }


    void DataTreeBuilder::update(QTreeWidgetItem* item, const std::string& key, aron::datanavigator::Navigator& data)
    {
        this->setRowTexts(item, key, data);

        if (auto cast = dynamic_cast<aron::datanavigator::DictNavigator*>(&data))
        {
            updateTree(item, *cast);
        }
        else if (auto cast = dynamic_cast<aron::datanavigator::ListNavigator*>(&data))
        {
            updateTree(item, *cast);
        }
    }


}
