/*
 * This file is part of ArmarX.
 *
 * Copyright (C) 2012-2016, High Performance Humanoid Technologies (H2T),
 * Karlsruhe Institute of Technology (KIT), all rights reserved.
 *
 * ArmarX is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License version 2 as
 * published by the Free Software Foundation.
 *
 * ArmarX is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program. If not, see <http://www.gnu.org/licenses/>.
 *
 * @author     Fabian Peller-Konrad (fabian dot peller-konrad at kit dot edu)
 * @copyright  http://www.gnu.org/licenses/gpl-2.0.txt
 *             GNU General Public License
 */

#pragma once

// STD/STL
#include <memory>
#include <typeindex>
#include <typeinfo>
#include <map>
#include <string>

// ArmarX
#include <RobotAPI/libraries/aron/core/Config.h>

namespace armarx::aron::type
{
    enum class Descriptor
    {
#define RUN_ARON_MACRO(upperType, lowerType, capsType) \
    e##upperType,

        HANDLE_ALL_ARON_TYPES
#undef RUN_ARON_MACRO
        eUnknown = -1
    };

    const std::map<type::Descriptor, std::string> _descriptorstring =
    {
#define RUN_ARON_MACRO(upperType, lowerType, capsType) \
    { Descriptor::e##upperType, "armarx::aron::type::Descriptor::e" + std::string(#upperType) },

        HANDLE_ALL_ARON_TYPES
#undef RUN_ARON_MACRO
        {Descriptor::eUnknown, "armarx::aron::type::Descriptor::eUnknown"}
    };

    inline std::string DESCRIPTOR_TO_STRING(const type::Descriptor d)
    {
        return type::_descriptorstring.at(d);
    }
}

namespace armarx::aron::data
{
    enum class Descriptor
    {
#define RUN_ARON_MACRO(upperType, lowerType, capsType) \
    e##upperType,

        HANDLE_ALL_ARON_DATA
#undef RUN_ARON_MACRO
        eUnknown = -1
    };

    const std::map<data::Descriptor, std::string> _descriptorstring =
    {
#define RUN_ARON_MACRO(upperType, lowerType, capsType) \
    { Descriptor::e##upperType, "armarx::aron::data::Descriptor::e" + std::string(#upperType) },

        HANDLE_ALL_ARON_DATA
#undef RUN_ARON_MACRO
        {Descriptor::eUnknown, "armarx::aron::data::Descriptor::eUnknown"}
    };

    inline std::string DESCRIPTOR_TO_STRING(const data::Descriptor d)
    {
        return data::_descriptorstring.at(d);
    }
}
