﻿/*
* This file is part of ArmarX.
*
* ArmarX is free software; you can redistribute it and/or modify
* it under the terms of the GNU General Public License version 2 as
* published by the Free Software Foundation.
*
* ArmarX is distributed in the hope that it will be useful, but
* WITHOUT ANY WARRANTY; without even the implied warranty of
* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
* GNU General Public License for more details.
*
* You should have received a copy of the GNU General Public License
* along with this program. If not, see <http://www.gnu.org/licenses/>.
*
* @author     Fabian Peller (fabian dot peller at kit dot edu)
* @copyright  http://www.gnu.org/licenses/gpl-2.0.txt
*             GNU General Public License
*/

#pragma once

// STD/STL
#include <memory>
#include <string>

// ArmarX
#include <RobotAPI/interface/aron.h>
#include <RobotAPI/libraries/aron/core/Exception.h>
#include <RobotAPI/libraries/aron/core/Config.h>


namespace armarx::aron::typeIO
{
    class ReaderInterface;
    typedef std::shared_ptr<ReaderInterface> ReaderInterfacePtr;

    class ReaderInterface
    {
    public:
        virtual std::tuple<std::string, int> readStartObject() = 0;
        virtual bool readEndObject() = 0;
        virtual int readStartList() = 0;
        virtual bool readEndList() = 0;
        virtual int readStartDict() = 0;
        virtual bool readEndDict() = 0;
        virtual int readStartTuple() = 0;
        virtual bool readEndTuple() = 0;
        virtual int readStartPair() = 0;
        virtual bool readEndPair() = 0;

        virtual std::tuple<std::vector<int>, std::string> readEigenMatrix() = 0;
        virtual std::string readEigenQuaternion() = 0;
        virtual std::tuple<unsigned int, unsigned int, std::string> readIVTCByteImage() = 0;
        virtual std::tuple<std::vector<int>, std::string> readOpenCVMat() = 0;
        virtual std::tuple<unsigned int, unsigned int, std::string> readPCLPointCloud() = 0;
        virtual void readPosition() = 0;
        virtual void readOrientation() = 0;
        virtual void readPose() = 0;

        virtual void readInt() = 0;
        virtual void readLong() = 0;
        virtual void readFloat() = 0;
        virtual void readDouble() = 0;
        virtual void readString() = 0;
        virtual void readBool() = 0;
        virtual void readTime() = 0;

        virtual std::string readKey() = 0;
        virtual void loadMember(const std::string&) = 0;

        // Helper functions
        virtual type::Descriptor getTypeOfNext() const = 0;
    };
}
