/*
* This file is part of ArmarX.
*
* ArmarX is free software; you can redistribute it and/or modify
* it under the terms of the GNU General Public License version 2 as
* published by the Free Software Foundation.
*
* ArmarX is distributed in the hope that it will be useful, but
* WITHOUT ANY WARRANTY; without even the implied warranty of
* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
* GNU General Public License for more details.
*
* You should have received a copy of the GNU General Public License
* along with this program. If not, see <http://www.gnu.org/licenses/>.
*
* @author     Fabian Peller (fabian dot peller at kit dot edu)
* @copyright  http://www.gnu.org/licenses/gpl-2.0.txt
*             GNU General Public License
*/

#pragma once

// STD/STL
#include <memory>
#include <string>

// ArmarX
#include <RobotAPI/interface/aron.h>
#include <RobotAPI/libraries/aron/core/Exception.h>
#include <RobotAPI/libraries/aron/core/Config.h>



namespace armarx::aron::typeIO
{
    class WriterInterface;
    typedef std::shared_ptr<WriterInterface> WriterInterfacePtr;

    class WriterInterface
    {
    public:
        virtual void writeStartObject(const std::string&) = 0;
        virtual void writeEndObject() = 0;
        virtual void writeStartList() = 0;
        virtual void writeEndList() = 0;
        virtual void writeStartDict() = 0;
        virtual void writeEndDict() = 0;
        virtual void writeStartTuple() = 0;
        virtual void writeEndTuple() = 0;
        virtual void writeStartPair() = 0;
        virtual void writeEndPair() = 0;

        virtual void writeEigenMatrix(const std::vector<int>&, const std::string&) = 0;
        virtual void writeEigenQuaternion(const std::string&) = 0;
        virtual void writeIVTCByteImage(unsigned int w, unsigned int h, const std::string&) = 0;
        virtual void writeOpenCVMat(const std::vector<int>&, const std::string&) = 0;
        virtual void writePCLPointCloud(unsigned int w, unsigned int h, const std::string&) = 0;
        virtual void writePosition() = 0;
        virtual void writeOrientation() = 0;
        virtual void writePose() = 0;

        virtual void writeInt() = 0;
        virtual void writeLong() = 0;
        virtual void writeFloat() = 0;
        virtual void writeDouble() = 0;
        virtual void writeString() = 0;
        virtual void writeBool() = 0;
        virtual void writeTime() = 0;

        virtual void writeKey(const std::string&) = 0;
    };
}
