/*
 * This file is part of ArmarX.
 *
 * Copyright (C) 2012-2016, High Performance Humanoid Technologies (H2T),
 * Karlsruhe Institute of Technology (KIT), all rights reserved.
 *
 * ArmarX is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License version 2 as
 * published by the Free Software Foundation.
 *
 * ArmarX is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program. If not, see <http://www.gnu.org/licenses/>.
 *
 * @author     Fabian Peller-Konrad (fabian dot peller-konrad at kit dot edu)
 * @copyright  http://www.gnu.org/licenses/gpl-2.0.txt
 *             GNU General Public License
 */

#pragma once

// STD/STL
#include <string>
#include <map>

// Base Class
#include <RobotAPI/libraries/aron/core/navigator/type/Navigator.h>

namespace armarx::aron::typenavigator
{
    class TupleNavigator;
    typedef std::shared_ptr<TupleNavigator> TupleNavigatorPtr;

    class TupleNavigator :
        virtual public Navigator
    {
    public:
        using PointerType = TupleNavigatorPtr;

    public:
        // constructors
        TupleNavigator(const Path& path = Path());
        TupleNavigator(const type::AronTuplePtr&, const Path& path = Path());

        std::vector<NavigatorPtr> getAcceptedTypes() const;
        NavigatorPtr getAcceptedType(unsigned int i) const;
        void addAcceptedType(const NavigatorPtr&);

        type::AronTuplePtr toAronTuplePtr() const;

        // static methods
        static TupleNavigatorPtr DynamicCast(const NavigatorPtr& n);
        static TupleNavigatorPtr DynamicCastAndCheck(const NavigatorPtr& n);

        // virtual implementations
        virtual std::vector<NavigatorPtr> getChildren() const override;
        virtual size_t childrenSize() const override;
        virtual type::AronTypePtr getResult() const override;
        virtual std::string getName() const override;

    private:
        std::vector<NavigatorPtr> acceptedTypes;
        type::AronTuplePtr type;
    };
}
