/*
 * This file is part of ArmarX.
 *
 * Copyright (C) 2012-2016, High Performance Humanoid Technologies (H2T),
 * Karlsruhe Institute of Technology (KIT), all rights reserved.
 *
 * ArmarX is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License version 2 as
 * published by the Free Software Foundation.
 *
 * ArmarX is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program. If not, see <http://www.gnu.org/licenses/>.
 *
 * @author     Fabian Peller-Konrad (fabian dot peller-konrad at kit dot edu)
 * @copyright  http://www.gnu.org/licenses/gpl-2.0.txt
 *             GNU General Public License
 */

// Header
#include "Pose.h"

namespace armarx::aron::typenavigator
{
    // constructors
    PoseNavigator::PoseNavigator(const Path& path) :
        aron::Navigator<type::Descriptor, type::AronType>::Navigator(type::Descriptor::ePose, path),
        Navigator(type::Descriptor::ePose, path),
        type(new type::AronPose())
    {
    }

    PoseNavigator::PoseNavigator(const type::AronPosePtr& o, const Path& path) :
        aron::Navigator<type::Descriptor, type::AronType>::Navigator(type::Descriptor::ePose, path),
        Navigator(type::Descriptor::ePose, path),
        type(o)
    {
        CheckAronPtrForNull("AronPoseTypeNavigator", "AronPoseTypeNavigator", getPath(), o);
    }

    type::AronPosePtr PoseNavigator::toAronPosePtr() const
    {
        return type;
    }

    std::vector<int> PoseNavigator::getDimensions() const
    {
        return ACCEPTED_DIMENSION;
    }

    std::string PoseNavigator::getTypename() const
    {
        return ACCEPTED_TYPE;
    }

    // static methods
    PoseNavigatorPtr PoseNavigator::DynamicCast(const NavigatorPtr& n)
    {
        return std::dynamic_pointer_cast<PoseNavigator>(n);
    }

    PoseNavigatorPtr PoseNavigator::DynamicCastAndCheck(const NavigatorPtr& n)
    {
        CheckTypeNavigatorPtrForNull("AronPoseTypeNavigator", "DynamicCast[Before]", n);
        PoseNavigatorPtr casted = PoseNavigator::DynamicCast(n);
        CheckTypeNavigatorPtrForNull("AronPoseTypeNavigator", "DynamicCast[After]", n->getPath(), casted);
        return casted;
    }

    // virtual implementations
    std::vector<NavigatorPtr> PoseNavigator::getChildren() const
    {
        return {};
    }

    size_t PoseNavigator::childrenSize() const
    {
        return 0;
    }

    type::AronTypePtr PoseNavigator::getResult() const
    {
        return toAronPosePtr();
    }

    std::string PoseNavigator::getName() const
    {
        return "AronPoseType";
    }
}

